(function ($) {
  /**
   * Determines if the dialog is active or not using "aria-expanded" attribute.
   *
   * @param element
   * @returns {boolean}
   */
  function isActive(element) {
    return element.attr('aria-expanded') === 'true';
  }

  /**
   * Extracts and returns content identifier from the material HTML.
   *
   * @returns {string}
   * @throws {Error}
   */
  function getContentId() {
    var elements = $(document).find('.h5p-content');

    if (elements.length !== 1) {
      throw new Error('Unable to find the content identifier!');
    }

    return elements.data('content-id');
  }

  /**
   * Extracts and returns content data from H5PIntegration.
   *
   * @param contentId
   * @returns {object}
   * @throws {Error}
   */
  function getContentData(contentId) {
    var cid = 'cid-' + contentId;

    if (!H5PIntegration.contents.hasOwnProperty(cid)) {
      throw new Error('Unable to locate the content data for content ' + contentId + '!');
    }

    return H5PIntegration.contents[cid];
  }

  /**
   * Create and trigger an xAPI "opened-tip" event.
   *
   * @param text
   */
  function sendOpenedTipEvent(text) {
    var contentId, contentData;

    try {
      contentId = getContentId();
      contentData = getContentData(contentId);
    } catch (e) {
      console.error(e);
    }

    eventDispatcher.triggerXAPI({
      id: 'https://h5p.ee/xapi/verbs/opened-tip',
      display: {
        'en-US': 'opened tip'
      }
    }, {
      object: {
        id: contentData.url,
        objectType: 'Activity',
        definition: {
          extensions: {
            'http://h5p.org/x-api/h5p-local-content-id': contentId,
            'https://h5p.ee/xapi/extensions/tip-text': text
          },
          name: {
            'en-US': contentData.metadata.title
          }
        }
      },
      context: {
        contextActivities: {
          category: [
            {
              id: 'http://h5p.org/libraries/' + contentData.library.replace(' ', '-'),
              objectType: 'Activity'
            }
          ]
        }
      }
    });
  }

  // A local EventDispatcher instance is required because using H5P.externalDispatcher directly results in two events
  var eventDispatcher = new H5P.EventDispatcher();

  H5P.JoubelUI.createTipOriginal = H5P.JoubelUI.createTip;
  H5P.JoubelUI.createTip = function (text, params) {
    var element = H5P.JoubelUI.createTipOriginal(text, params);

    if (element) {
      element.on('click', function () {
        if (isActive(element)) {
          sendOpenedTipEvent(text);
        }
      });
      element.on('keydown', function (event) {
        if (event.which === 32 || event.which === 13) { // Space & enter key
          if (isActive(element)) {
            sendOpenedTipEvent(text);
          }
        }
      });
    }

    return element;
  };
})(H5P.jQuery);